@@ -11,3 +11,10 @@
 #undef HAVE_GTK_SIGNAL_HANDLER_PENDING
 #undef HAVE_PROGRAM_INVOCATION_SHORT_NAME
 #undef HAVE_PROGRAM_INVOCATION_NAME
+#undef HAVE_SYS_ERRLIST
+#undef HAVE_LIBJPEG
+#undef HAVE_LIBGIF
+#undef HAVE_LIBTIFF
+#undef HAVE_LIBZ
+#undef HAVE_LIBPNG
+#undef GNOME_ENABLE_DEBUG
