@@ -1,14 +1,14 @@
 #ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/XmHTML.c,v 1.22 1998/02/21 00:13:13 unammx Exp $";
+static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/XmHTML.c,v 1.25 1998/05/09 17:05:20 kmaraas Exp $";
 #endif
 /*****
 * XmHTML.c : XmHTML main routines
 *
-* This file Version	$Revision: 1.22 $
+* This file Version	$Revision: 1.25 $
 *
 * Creation date:		Thu Nov 21 05:02:44 GMT+0100 1996
-* Last modification: 	$Date: 1998/02/21 00:13:13 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1998/05/09 17:05:20 $
+* By:					$Author: kmaraas $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -35,6 +35,15 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/XmHTML
 /*****
 * ChangeLog 
 * $Log: XmHTML.c,v $
+* Revision 1.25  1998/05/09 17:05:20  kmaraas
+* 1998-05-09  Kjartan Maraas  <kmaraas@fib.hl.no>
+*
+* 	* XmHTML.c: Added declaration of DestroyPhaseZero to get
+* 	rid of compiler warning.
+*
+* Revision 1.24  1998/04/08 14:45:35  jpaint
+* Reverted border changes (finally, sorry miguel!)
+*
 * Revision 1.22  1998/02/21 00:13:13  unammx
 * Fri Feb 20 18:14:15 1998  Miguel de Icaza  <miguel@nuclecu.unam.mx>
 *
@@ -348,6 +357,8 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/XmHTML
 * _XmRedisplayGadgets (Widget, XEvent*, Region);
 */
 
+static void DestroyPhaseZero (XmHTMLWidget html);
+
 /*** Public Variable Declarations ***/
 
 /*** Private Datatype Declarations ****/
