@@ -29,6 +29,11 @@
 #include <glib.h>
 #include "libgnome.h"
 
+#ifndef HAVE_STRNDUP
+/* Like strdup, but only copy N chars.  */
+extern char *strndup (const char *s, size_t n);
+#endif
+
 #define STRSIZE 4096
 #define overflow (next == &CharBuffer [STRSIZE-1])
 
@@ -75,21 +80,14 @@ typedef struct {
 	char *path, *opath;
 } ParsedPath;
 
-struct _prefix_list {
-	char *p_prefix;
-	struct _prefix_list *p_back;
-};
-
-typedef struct _prefix_list prefix_list_t;
-
 /*
  * Prefix for all the configuration operations
  * iff the path does not begin with / or with #
  */
 
-#define prefix (prefix_list ? prefix_list->p_prefix : NULL)
+#define prefix (prefix_list ? prefix_list->data : NULL)
 
-static prefix_list_t *prefix_list;
+static GSList *prefix_list;
 
 static TProfile *Current = 0;
 
@@ -106,7 +104,7 @@ release_path (ParsedPath *p)
 }
 
 static ParsedPath *
-parse_path (const char *path)
+parse_path (const char *path, gint priv)
 {
 	ParsedPath *p = g_malloc (sizeof (ParsedPath));
 	char *sep;
@@ -162,7 +160,12 @@ parse_path (const char *path)
 		}
 		if (*p->file == '/')
 			p->file++;
-		p->file = g_concat_dir_and_file (gnome_user_dir, p->file);
+		if(priv)
+			p->file = g_concat_dir_and_file (gnome_user_private_dir,
+							 p->file);
+		else
+			p->file = g_concat_dir_and_file (gnome_user_dir,
+							 p->file);
 	}
 	return p;
 }
@@ -221,6 +224,9 @@ escape_string_and_dup (char *s)
 {
 	char *return_value, *p = s;
 	int len = 0;
+
+	if(!s)
+		return g_strdup("");
 	
 	while (*p){
 		len++;
@@ -337,7 +343,7 @@ load (const char *file)
 				SecHeader->keys->value = decode_string_and_dup (CharBuffer);
 				state = c == '\n' ? KeyDef : IgnoreToEOL;
 				next = CharBuffer;
-#ifdef DEBUG
+#ifdef GNOME_ENABLE_DEBUG
 				printf ("[%s] (%s)=%s\n", SecHeader->section_name,
 					SecHeader->keys->key_name, SecHeader->keys->value);
 #endif
@@ -457,6 +463,89 @@ dump_sections (FILE *profile, TSecHeader *p)
 	}
 }
 
+/*check the path and if we need to create directories create them with
+  mode newmode, it needs an absolute path name or it will fail, it
+  needs to be passed the dir and the filename since it will take the
+  filename off*/
+static gint
+check_path(char *path, mode_t newmode)
+{
+	gchar *dirpath;
+	gchar *p;
+	GString *newpath;
+	struct stat s;
+
+
+	g_return_val_if_fail(path!=NULL, FALSE);
+
+	if(strchr(path,'/')==NULL)
+		return FALSE;
+	dirpath = g_strdup(path);
+	if(!dirpath)
+		return FALSE;
+
+	p = strrchr(dirpath,'/');
+		*p='\0';
+
+	if(*dirpath == '\0') {
+		g_free(dirpath);
+		return FALSE;
+	}
+
+	/*not absolute, we refuse to work*/
+	if(dirpath[0]!='/') {
+		g_free(dirpath);
+		return FALSE;
+	}
+
+	/*special case if directory exists, this is probably gonna happen
+	  a lot so we don't want to go though checking it part by part*/
+	if(stat(dirpath,&s)==0) {
+		g_free(dirpath);
+		/*check if a directory*/
+		if(!S_ISDIR(s.st_mode))
+			return FALSE;
+		else
+			return TRUE;
+	}
+
+
+	/*skip leading '/'*/
+	p = dirpath;
+	while(*p == '/')
+		p++;
+
+	p=strtok(p,"/");
+	newpath = g_string_new("");
+	do {
+		newpath = g_string_append_c(newpath,'/');
+		newpath = g_string_append(newpath,p);
+		if(stat(newpath->str,&s)==0) {
+			/*check if a directory*/
+			if(!S_ISDIR(s.st_mode)) {
+				g_free(dirpath);
+				g_string_free(newpath,TRUE);
+				return FALSE;
+			}
+		} else {
+			/*we couldn't stat it .. let's try making the
+			  directory*/
+			if(mkdir(newpath->str,newmode)!=0) {
+				/*error, return false*/
+				g_free(dirpath);
+				g_string_free(newpath,TRUE);
+				return FALSE;
+			}
+		}
+
+	} while((p=strtok(NULL,"/"))!=NULL);
+
+	g_free(dirpath);
+	g_string_free(newpath,TRUE);
+
+	return TRUE;
+}
+
 static void 
 dump_profile (TProfile *p)
 {
@@ -468,7 +557,8 @@ dump_profile (TProfile *p)
 
 	/* .ado: p->filename can be empty, it's better to jump over */
 	if (p->filename[0] != (char) 0)
-		if ((profile = fopen (p->filename, "w")) != NULL){
+		if (check_path(p->filename,0755) &&
+		    (profile = fopen (p->filename, "w")) != NULL){
 			dump_sections (profile, p->section);
 			fclose (profile);
 		}
@@ -519,7 +609,7 @@ free_profile (TProfile *p)
 }
 
 void 
-gnome_config_clean_file (const char *path)
+_gnome_config_clean_file (const char *path, gint priv)
 {
 	TProfile *p;
 	ParsedPath *pp;
@@ -529,7 +619,7 @@ gnome_config_clean_file (const char *path)
 		return;
 
 	fake_path = g_copy_strings (path, "/section/key", NULL);
-	pp = parse_path (fake_path);
+	pp = parse_path (fake_path, priv);
 	g_free (fake_path);
 	
 	for (p = Base; p; p = p->link){
@@ -546,7 +636,7 @@ gnome_config_clean_file (const char *path)
 }
 
 void *
-gnome_config_init_iterator (const char *path)
+_gnome_config_init_iterator (const char *path, gint priv)
 {
 	TProfile   *New;
 	TSecHeader *section;
@@ -556,7 +646,7 @@ gnome_config_init_iterator (const char *path)
 
 
 	fake_path = g_copy_strings (path, "/key", NULL);
-	pp = parse_path (fake_path);
+	pp = parse_path (fake_path, priv);
 	g_free (fake_path);
 	
 	if (!is_loaded (pp->file, &section)){
@@ -586,7 +676,7 @@ gnome_config_init_iterator (const char *path)
 }
 
 void *
-gnome_config_init_iterator_sections (const char *path)
+_gnome_config_init_iterator_sections (const char *path, gint priv)
 {
 	TProfile   *New;
 	TSecHeader *section;
@@ -596,7 +686,7 @@ gnome_config_init_iterator_sections (const char *path)
 
 
 	fake_path = g_copy_strings (path, "/section/key", NULL);
-	pp = parse_path (fake_path);
+	pp = parse_path (fake_path, priv);
 	g_free (fake_path);
 	
 	if (!is_loaded (pp->file, &section)){
@@ -654,21 +744,29 @@ gnome_config_iterator_next (void *s, char **key, char **value)
 }
 
 void 
-gnome_config_clean_section (const char *path)
+_gnome_config_clean_section (const char *path, gint priv)
 {
+	TProfile   *New;
 	TSecHeader *section;
 	ParsedPath *pp;
 	char *fake_path;
 
 	fake_path = g_copy_strings (path, "/key", NULL);
-	pp = parse_path (fake_path);
+	pp = parse_path (fake_path, priv);
 	g_free (fake_path);
 	
-	/* We assume the user has called one of the other initialization funcs */
 	if (!is_loaded (pp->file, &section)){
-		fprintf (stderr,"Warning: profile_clean_section called before init\n");
-		release_path (pp);
-		return;
+		struct stat st;
+		if (stat (pp->file, &st) == -1) st.st_mtime = 0;
+
+		New = (TProfile *) g_malloc (sizeof (TProfile));
+		New->link = Base;
+		New->filename = g_strdup (pp->file);
+		New->section = load (pp->file);
+		New->mtime = st.st_mtime;
+		Base = New;
+		section = New->section;
+		Current = New;
 	}
 	/* We only disable the section, so it will still be g_freed, but it */
 	/* won't be find by further walks of the structure */
@@ -682,24 +780,32 @@ gnome_config_clean_section (const char *path)
 }
 
 void 
-gnome_config_clean_key (const char *path)
+_gnome_config_clean_key (const char *path, gint priv)
 	/* *section_name, char *file */
 {
+	TProfile   *New;
 	TSecHeader *section;
 	TKeys *key;
 	ParsedPath *pp;
 	
-	pp = parse_path (path);
+	pp = parse_path (path, priv);
 	
-	/* We assume the user has called one of the other initialization funcs */
 	if (!is_loaded (pp->file, &section)){
-		fprintf (stderr,"Warning: profile_clean_section called before init\n");
-		release_path (pp);
-		return;
+		struct stat st;
+		if (stat (pp->file, &st) == -1) st.st_mtime = 0;
+
+		New = (TProfile *) g_malloc (sizeof (TProfile));
+		New->link = Base;
+		New->filename = g_strdup (pp->file);
+		New->section = load (pp->file);
+		New->mtime = st.st_mtime;
+		Base = New;
+		section = New->section;
+		Current = New;
 	}
 	for (; section; section = section->link){
-		if (strcasecmp (section->section_name, pp->section))
-			continue;
+	        if (strcasecmp (section->section_name, pp->section))
+		        continue;
 		for (key = section->keys; key; key = key->link){
 			if (strcasecmp (key->key_name, pp->key))
 				continue;
@@ -710,21 +816,30 @@ gnome_config_clean_key (const char *path)
 }
 
 gboolean 
-gnome_config_has_section (const char *path)
+_gnome_config_has_section (const char *path, gint priv)
 	/* char *section_name, char *profile */
 {
+	TProfile   *New;
 	TSecHeader *section;
 	ParsedPath *pp;
 	char *fake_path;
 
 	fake_path = g_copy_strings (path, "/key", NULL);
-	pp = parse_path (fake_path);
+	pp = parse_path (fake_path,priv);
 	g_free (fake_path);
 	
-	/* We assume the user has called one of the other initialization funcs */
 	if (!is_loaded (pp->file, &section)){
-		release_path (pp);
-		return 0;
+		struct stat st;
+		if (stat (pp->file, &st) == -1) st.st_mtime = 0;
+
+		New = (TProfile *) g_malloc (sizeof (TProfile));
+		New->link = Base;
+		New->filename = g_strdup (pp->file);
+		New->section = load (pp->file);
+		New->mtime = st.st_mtime;
+		Base = New;
+		section = New->section;
+		Current = New;
 	}
 	for (; section; section = section->link){
 		if (strcasecmp (section->section_name, pp->section))
@@ -740,16 +855,17 @@ void
 gnome_config_drop_all (void)
 {
 	free_profile (Base);
+	Base = 0;
 }
 
 gint
-gnome_config_get_int_with_default (const char *path, gboolean *def)
+_gnome_config_get_int_with_default (const char *path, gboolean *def, gint priv)
 {
 	ParsedPath *pp;
 	const char *r;
 	int  v;
 	
-	pp = parse_path (path);
+	pp = parse_path (path, priv);
 	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file,
 			   def);
 
@@ -761,13 +877,77 @@ gnome_config_get_int_with_default (const char *path, gboolean *def)
 }
 
 char *
-gnome_config_get_string_with_default (const char *path, gboolean *def)
+_gnome_config_get_translated_string_with_default (const char *path,
+						  gboolean *def,
+						  gint priv)
+{
+  GList *language_list;
+
+  char *value= NULL;
+
+  language_list= gnome_i18n_get_language_list ("LC_ALL");
+  
+  while (!value && language_list)
+    {
+      const char *lang= language_list->data;
+
+      if (strcmp (lang, "C") == 0)
+	{
+	  value = _gnome_config_get_string_with_default (path, def, priv);
+	  if (value)
+	    return value;
+	  else
+	    {
+	      /* FIXME: this "else" block should go away in some time.
+	       * We use it to handle the old broken files that were
+	       * written by the old buggy set_translated_string
+	       * function, which *did* append the "[C]" suffix.
+	       */
+
+	      gchar *tkey;
+
+	      tkey = g_copy_strings (path, "[C]", NULL);
+	      value = _gnome_config_get_string_with_default (tkey, def, priv);
+	      g_free (tkey);
+
+	      if (!value || *value == '\0')
+		{
+		  g_free (value);
+		  value = NULL;
+		}
+
+	      return value;
+	    }
+	}
+      else
+	{
+	  gchar *tkey;
+	  
+	  tkey= g_copy_strings (path, "[", lang, "]", NULL);
+	  value= _gnome_config_get_string_with_default (tkey, def, priv);
+	  g_free (tkey);
+	  
+	  if (!value || *value == '\0')
+	    {
+	      g_free (value);
+	      value= NULL;
+	    }
+	}
+      language_list= language_list->next;
+    }
+
+  return value;
+}
+
+char *
+_gnome_config_get_string_with_default (const char *path, gboolean *def,
+				       gint priv)
 {
 	ParsedPath *pp;
 	const char *r;
 	char *ret = NULL;
 	
-	pp = parse_path (path);
+	pp = parse_path (path, priv);
 	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file,
 			   def);
 	if (r)
@@ -777,13 +957,14 @@ gnome_config_get_string_with_default (const char *path, gboolean *def)
 }
 
 gboolean
-gnome_config_get_bool_with_default (const char *path, gboolean *def)
+_gnome_config_get_bool_with_default (const char *path, gboolean *def,
+				     gint priv)
 {
 	ParsedPath *pp;
 	const char *r;
 	int  v;
 	
-	pp = parse_path (path);
+	pp = parse_path (path, priv);
 	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file,
 			   def);
 
@@ -802,77 +983,116 @@ gnome_config_get_bool_with_default (const char *path, gboolean *def)
 }
 
 void
-gnome_config_get_vector_with_default (const char *path, int *argcp,
-				      char ***argvp, gboolean *def)
+gnome_config_make_vector (const char *rr, int *argcp, char ***argvp)
 {
-	ParsedPath *pp;
-	const char *r, *p, *last;
-	char *tmp;
-	int  v;
-	int count;
+	char *p;
+	int count, esc_spcs;
+	int space_seen;
 
-	pp = parse_path (path);
-	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file,
-			   def);
-
-	g_return_if_fail(r != NULL);
-
-	/* Figure out how large to make return vector.  Start at 1
-	   because we want to make NULL-terminated array.  */
-	count = 1;
-	for (p = r; *p; ++p) {
+	/* Figure out how large to make return vector.  Start at 2
+	 * because we want to make NULL-terminated array, and because
+	 * the loop doesn't count the final element.
+	 */
+	count = 2;
+	space_seen = 0;
+	for (p = (char *) rr; *p; ++p) {
+		/* The way that entries are constructed by
+		   gnome_config_set_vector ensures we'll never see an
+		   unpaired `\' at the end of a string.  So this is
+		   safe.  */
 	        if (*p == '\\') {
-		        if (! *p)
-			        break;
 			++p;
 		} else if (*p == ' ') {
-		        ++count;
+			space_seen = 1;
+		} else if (space_seen){
+			count++;
+			space_seen = 0;
 		}
 	}
 
 	*argcp = count - 1;
-	*argvp = (char **) g_malloc (count * sizeof (char *));
-	(*argvp)[count - 1] = NULL;
+	*argvp = (char **) g_malloc0 (count * sizeof (char *));
 
+	p = (char *) rr;
 	count = 0;
-	last = r;
-	for (p = r; *p; ++p) {
-	        if (*p == '\\') {
-		        if (! *p)
-			        break;
-			++p;
-		} else if (*p == ' ') {
-		        tmp = g_malloc (p - last + 1);
-			strncpy (tmp, last, p - last);
-			tmp[p - last] = '\0';
-		        (*argvp)[count++] = tmp;
-			last = p + 1;
-		}
-	}
+	do {
+		char *tmp = p;
 
+		esc_spcs = 0;
+		while (*p && (esc_spcs ? 1 : (*p != ' '))){
+			esc_spcs = 0;
+			if (*p == '\\')
+				esc_spcs = 1;
+			p++;
+		}
+
+ 		(*argvp)[count++] = (char *) strndup (tmp, p - tmp);
+
+		while (*p && *p == ' ')
+			p++;
+	} while (*p);
+}
+
+void
+_gnome_config_get_vector_with_default (const char *path, int *argcp,
+				       char ***argvp, gboolean *def, gint priv)
+{
+	ParsedPath *pp;
+	const char *rr;
+	
+	pp = parse_path (path, priv);
+	rr = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file, def);
+
+	if (rr == NULL) {
+		*argvp = NULL;
+		*argcp = 0;
+	} else
+		gnome_config_make_vector (rr, argcp, argvp);
 	release_path (pp);
 }
 
 void
-gnome_config_set_string (const char *path, const char *new_value)
+_gnome_config_set_translated_string (const char *path, const char *value,
+				     gint priv)
+{
+  GList *language_list;
+  const char *lang;
+  char *tkey;
+
+  language_list= gnome_i18n_get_language_list("LC_ALL");
+
+  lang= language_list ? language_list->data : NULL;
+  
+  if (lang && (strcmp (lang, "C") != 0))
+    {
+      tkey = g_copy_strings (path, "[", lang, "]", NULL);
+      _gnome_config_set_string(tkey, value, priv);
+      g_free (tkey);
+    }
+  else
+    _gnome_config_set_string (path, value, priv);
+}
+
+void
+_gnome_config_set_string (const char *path, const char *new_value, gint priv)
 {
 	ParsedPath *pp;
 	const char *r;
 	
-	pp = parse_path (path);
+	pp = parse_path (path, priv);
 	r = access_config (SET, pp->section, pp->key, new_value, pp->file,
 			   NULL);
 	release_path (pp);
 }
 
 void
-gnome_config_set_int (const char *path, int new_value)
+_gnome_config_set_int (const char *path, int new_value, gint priv)
 {
 	ParsedPath *pp;
 	char intbuf [40];
 	const char *r;
 	
-	pp = parse_path (path);
+	pp = parse_path (path, priv);
 	sprintf (intbuf, "%d", new_value);
 	r = access_config (SET, pp->section, pp->key, intbuf, pp->file,
 			   NULL);
@@ -880,22 +1100,20 @@ gnome_config_set_int (const char *path, int new_value)
 }
 
 void
-gnome_config_set_bool (const char *path, gboolean new_value)
+_gnome_config_set_bool (const char *path, gboolean new_value, gint priv)
 {
 	ParsedPath *pp;
 	const char *r;
 	
-	pp = parse_path (path);
+	pp = parse_path (path, priv);
 	r = access_config (SET, pp->section, pp->key,
 			   new_value ? "true" : "false", pp->file, NULL);
 	release_path (pp);
 }
 
-void
-gnome_config_set_vector (const char *path, int argc,
-			 const char *const argv[])
+char *
+gnome_config_assemble_vector (int argc, const char *const argv [])
 {
-	ParsedPath *pp;
 	char *value, *p;
 	const char *s;
 	int i, len;
@@ -910,7 +1128,7 @@ gnome_config_set_vector (const char *path, int argc,
 	p = value = g_malloc (len);
 	for (i = 0; i < argc; ++i) {
 		for (s = argv[i]; *s; ++s) {
-			if (*s == ' ')
+			if (*s == ' ' || *s == '\\')
 				*p++ = '\\';
 			*p++ = *s;
 		}
@@ -918,33 +1136,38 @@ gnome_config_set_vector (const char *path, int argc,
 	}
 	*p = '\0';
 
-	pp = parse_path (path);
-	access_config (SET, pp->section, pp->key, value, pp->file, NULL);
-	g_free (value);
+	return value;
+}
+
+void
+_gnome_config_set_vector (const char *path, int argc,
+			  const char *const argv[],
+			  gint priv)
+{
+	ParsedPath *pp;
+	char *s;
+
+	pp = parse_path (path, priv);
+	s = gnome_config_assemble_vector (argc, argv);
+	access_config (SET, pp->section, pp->key, s, pp->file, NULL);
+	g_free (s);
 	release_path (pp);
 }
 
 void
 gnome_config_push_prefix (const char *path)
 {
-	prefix_list_t *p = g_malloc (sizeof (prefix_list_t));
-
-	p->p_back = prefix_list;
-	p->p_prefix = g_strdup (path);
-	prefix_list = p;
+  prefix_list = g_slist_prepend(prefix_list, g_strdup(path));
 }
 
 void
 gnome_config_pop_prefix (void)
 {
-	prefix_list_t *p = prefix_list;
-	
-	if (!p)
-		return;
-
-	g_free (p->p_prefix);
-	prefix_list = p->p_back;
-	g_free (p);
+  if(prefix_list)
+    {
+      g_free(prefix_list->data);
+      prefix_list = g_slist_remove_link(prefix_list, prefix_list);
+    }
 }
 
 #ifdef TEST
@@ -955,7 +1178,7 @@ x (char *str, char *file, char *sec, char *key, char *val)
 	ParsedPath *pp;
 
 	printf ("%s\n", str);
-	pp = parse_path (str);
+	pp = parse_path (str, FALSE);
 	printf ("   file: %s [%s]\n", pp->file, file);
 	printf ("   sect: %s [%s]\n", pp->section, sec);
 	printf ("   key:  %s [%s]\n", pp->key, key);
@@ -975,3 +1198,128 @@ main ()
 	
 }
 #endif
+
+#ifndef AVOID_THIS_EXTRA_COOL_HACK
+
+/* THIS WILL BE GONE SOON (Can be removed at any time, nothing but old
+ * binaries depends on this).
+ */
+#undef gnome_config_get_string_with_default
+#undef gnome_config_get_translated_string_with_default
+#undef gnome_config_get_int_with_default
+#undef gnome_config_get_bool_with_default
+#undef gnome_config_get_vector_with_default
+
+char *
+gnome_config_get_string_with_default (const char *path, gboolean *def)
+{
+	return _gnome_config_get_string_with_default(path,def,FALSE);
+}
+
+char *
+gnome_config_get_translated_string_with_default (const char *path, gboolean *def)
+{
+	return _gnome_config_get_translated_string_with_default(path, def, FALSE);
+}
+
+gint
+gnome_config_get_int_with_default (const char *path, gboolean *def)
+{
+	return _gnome_config_get_int_with_default(path, def, FALSE);
+}
+
+gboolean
+gnome_config_get_bool_with_default (const char *path, gboolean *def)
+{
+	return _gnome_config_get_bool_with_default (path, def, FALSE);
+}
+
+void
+gnome_config_get_vector_with_default     (const char *path, gint *argcp, char ***argvp, gboolean *def)
+{
+        _gnome_config_get_vector_with_default (path, argcp, argvp, def, FALSE);
+}
+
+#undef gnome_config_set_string
+#undef gnome_config_set_translated_string
+#undef gnome_config_set_int
+#undef gnome_config_set_bool
+#undef gnome_config_set_vector
+
+void
+gnome_config_set_string (const char *path, const char *value)
+{
+	_gnome_config_set_string (path, value, FALSE);
+}
+
+void
+gnome_config_set_translated_string(const char *path, const char *val)
+{
+	
+	_gnome_config_set_translated_string (path, val, FALSE);
+}
+
+void
+gnome_config_set_int(const char *path, int value)
+{
+	_gnome_config_set_int (path, value, FALSE);
+}
+
+void
+gnome_config_set_bool(const char *path, gboolean value)
+{
+	_gnome_config_set_bool (path, value, FALSE);
+}
+
+void
+gnome_config_set_vector(const char *path, int argc, const char * const argv[])
+{
+	_gnome_config_set_vector (path, argc, argv, FALSE);
+}
+
+#undef gnome_config_clean_key
+#undef gnome_config_clean_section
+#undef gnome_config_clean_file
+#undef gnome_config_has_section
+#undef gnome_config_init_iterator
+#undef gnome_config_init_iterator_sections
+
+void
+gnome_config_clean_key(const char *path)
+{
+	_gnome_config_clean_key(path,FALSE);
+}
+
+void
+gnome_config_clean_section(const char *path)
+{
+	_gnome_config_clean_section(path,FALSE);
+}
+
+void
+gnome_config_clean_file(const char *path)
+{
+	_gnome_config_clean_file(path,FALSE);
+}
+
+gboolean
+gnome_config_has_section(const char *path)
+{
+	return _gnome_config_has_section(path,FALSE);
+}
+
+
+void *
+gnome_config_init_iterator(const char *path)
+{
+	return _gnome_config_init_iterator(path,FALSE);
+}
+
+
+void *
+gnome_config_init_iterator_sections(const char *path)
+{
+	return _gnome_config_init_iterator_sections(path,FALSE);
+}
+
+#endif
