@@ -17,95 +17,32 @@
 #include "gnome-util.h"
 #include "gnome-config.h"
 #include "gnome-dentry.h"
+#include "gnome-string.h"
+#include "gnome-exec.h"
 
-#define free_if_empty(x) { if (x) g_free (x); }
+/* g_free already checks if x is NULL */
+#define free_if_empty(x) g_free (x)
 
-int
+char *
 gnome_is_program_in_path (char *program)
 {
-	static char *path;
-	static char **paths;
+	static char **paths = NULL;
 	char **p;
 	char *f;
 	
-	if (!path){
-		char *p;
-		int i, pc = 1;
+	if (!paths)
+	  paths = gnome_string_split(getenv("PATH"), ":", -1);
 
-		path = g_strdup (getenv ("PATH"));
-		for (p = path; *p; p++)
-			if (*p == ':')
-				pc++;
-
-		paths = (char **) g_malloc (sizeof (char *) * (pc+1));
-
-		for (p = path, i = 0; i < pc; i++){
-			paths [i] = strtok (p, ":");
-			p = NULL;
-		}
-		paths [pc] = NULL;
-	}
 	p = paths;
 	while (*p){
 		f = g_concat_dir_and_file (*p, program);
-		if (g_file_exists (f)){
-			g_free (f);
-			return 1;
-		}
-		g_free (f);
+		if (g_file_exists(f))
+			return f;
+		g_free(f);
 		p++;
 	}
 	return 0;
 }
-
-/* Are the following two functions useful enough to be put into gnome-config? */
-
-static char *
-get_translated_string (char *key)
-{
-	/* FIXME: I don't know if getenv("LANG") is the right way to get the language */
-
-	char *lang;
-	char *tkey;
-	char *value;
-
-	lang = getenv ("LANG");
-
-	if (lang) {
-		tkey = g_copy_strings (key, "[", lang, "]", NULL);
-		value = gnome_config_get_string (tkey);
-		g_free (tkey);
-
-		if (value && *value == '\0') {
-			g_free (value);
-			value = NULL;
-		}
-
-		if (!value)
-			value = gnome_config_get_string (key); /* Fall back to untranslated case */
-	} else
-		value = gnome_config_get_string (key);
-
-	return value;
-}
-
-static void
-put_translated_string (char *key, char *string)
-{
-	/* FIXME: same as previous function */
-
-	char *lang;
-	char *tkey;
-
-	lang = getenv ("LANG");
-
-	if (lang) {
-		tkey = g_copy_strings (key, "[", lang, "]", NULL);
-		gnome_config_set_string (tkey, string);
-		g_free (tkey);
-	} else
-		gnome_config_set_string (key, string);
-}
 	      
 GnomeDesktopEntry *
 gnome_desktop_entry_load_flags (char *file, int clean_from_memory)
@@ -113,8 +50,11 @@ gnome_desktop_entry_load_flags (char *file, int clean_from_memory)
 	GnomeDesktopEntry *newitem;
 	char *prefix;
 	char *name, *type;
-	char *exec_file, *try_file;
+	char *try_file;
+	char **exec_vector;
+	int exec_length;
 	char *icon_base;
+	char *p;
 	
 	g_assert (file != NULL);
 
@@ -123,10 +63,10 @@ gnome_desktop_entry_load_flags (char *file, int clean_from_memory)
 	gnome_config_push_prefix (prefix);
 	g_free (prefix);
 
-	name = get_translated_string ("Name");
+	name = gnome_config_get_translated_string ("Name");
 	if (!name) {
 		gnome_config_pop_prefix ();
-		return 0;
+		return NULL;
 	}
 
 	/* FIXME: we only test for presence of Exec/TryExec keys if
@@ -136,43 +76,37 @@ gnome_desktop_entry_load_flags (char *file, int clean_from_memory)
 	 */
 
 	type      = gnome_config_get_string ("Type");
-	exec_file = gnome_config_get_string ("Exec");
+	gnome_config_get_vector ("Exec", &exec_length, &exec_vector);
 	try_file  = gnome_config_get_string ("TryExec");
+	p = 0;
 
-	if (!type || (strcmp (type, "Directory") != 0)) {
-		if (!exec_file) {
+	if (!type || (strcmp (type, "Directory") != 0)){
+		if(!exec_vector || (try_file && !(p = gnome_is_program_in_path(try_file)))){
+			free_if_empty (p);
 			free_if_empty (name);
 			free_if_empty (type);
+			gnome_string_array_free (exec_vector);
 			free_if_empty (try_file);
-
+			
 			gnome_config_pop_prefix ();
-			return 0;
-		}
-
-		if (try_file) {
-			if (!gnome_is_program_in_path (try_file)) {
-				free_if_empty (name);
-				free_if_empty (type);
-				free_if_empty (exec_file);
-				free_if_empty (try_file);
-
-				gnome_config_pop_prefix ();
-				return 0;
-			}
+			return NULL;
 		}
+		if (p)
+			g_free (p);
 	}
 	
 	newitem = g_new (GnomeDesktopEntry, 1);
 
-	newitem->name          = get_translated_string ("Name");
-	newitem->comment       = get_translated_string ("Comment");
-	newitem->exec          = exec_file;
+	newitem->name          = gnome_config_get_translated_string ("Name");
+	newitem->comment       = gnome_config_get_translated_string ("Comment");
+	newitem->exec_length   = exec_length;
+	newitem->exec          = exec_vector;
 	newitem->tryexec       = try_file;
 	newitem->docpath       = gnome_config_get_string ("DocPath");
 	newitem->terminal      = gnome_config_get_bool   ("Terminal=0");
 	newitem->type          = gnome_config_get_string ("Type");
 	newitem->geometry      = gnome_config_get_string ("Geometry");
-	newitem->multiple_args = gnome_config_get_bool ("MultipleArgs=0");
+	newitem->multiple_args = gnome_config_get_bool   ("MultipleArgs=0");
 	newitem->location      = g_strdup (file);
 	icon_base	       = gnome_config_get_string ("Icon");
 	
@@ -220,13 +154,14 @@ gnome_desktop_entry_save (GnomeDesktopEntry *dentry)
 	g_free (prefix);
 
 	if (dentry->name)
-		put_translated_string ("Name", dentry->name);
+		gnome_config_set_translated_string ("Name", dentry->name);
 
 	if (dentry->comment)
-		put_translated_string ("Comment", dentry->comment);
+		gnome_config_set_translated_string ("Comment", dentry->comment);
 
 	if (dentry->exec)
-		gnome_config_set_string ("Exec", dentry->exec);
+		gnome_config_set_vector ("Exec", dentry->exec_length,
+					 (const char * const *) dentry->exec);
 
 	if (dentry->tryexec)
 		gnome_config_set_string ("TryExec", dentry->tryexec);
@@ -234,6 +169,9 @@ gnome_desktop_entry_save (GnomeDesktopEntry *dentry)
 	if (dentry->icon)
 		gnome_config_set_string ("Icon", dentry->icon);
 
+	if (dentry->geometry)
+		gnome_config_set_string ("Geometry", dentry->geometry);
+	
 	if (dentry->docpath)
 		gnome_config_set_string ("DocPath", dentry->docpath);
 
@@ -250,31 +188,104 @@ gnome_desktop_entry_save (GnomeDesktopEntry *dentry)
 void
 gnome_desktop_entry_free (GnomeDesktopEntry *item)
 {
-	g_assert (item != NULL);
-	
-	free_if_empty (item->name);
-	free_if_empty (item->comment);
-	free_if_empty (item->exec);
-	free_if_empty (item->icon);
-	free_if_empty (item->docpath);
-	free_if_empty (item->type);
-	free_if_empty (item->location);
+  if(item)
+    {
+      free_if_empty (item->name);
+      free_if_empty (item->comment);
+      gnome_string_array_free (item->exec);
+      free_if_empty (item->icon);
+      free_if_empty (item->docpath);
+      free_if_empty (item->type);
+      free_if_empty (item->location);
+      free_if_empty (item->geometry);
+      g_free (item);
+    }
+}
 
-	g_free (item);
+void
+gnome_desktop_entry_launch_with_args (GnomeDesktopEntry *item, int the_argc, char *the_argv[])
+{
+	char **argv;
+	int i, argc;
+	char *uargv[4];
+
+	g_assert (item != NULL);
+
+	if (item->terminal) {
+		char **term_argv;
+		int term_argc;
+		char *xterm_argv[2];
+
+		gnome_config_get_vector ("/Gnome/Applications/Terminal",
+					 &term_argc, &term_argv);
+		if (term_argv == NULL) {
+			term_argc = 2;
+			term_argv = xterm_argv;
+			xterm_argv[0] = "xterm";
+			xterm_argv[1] = "-e";
+		}
+
+		argc = the_argc + term_argc + item->exec_length;
+		argv = (char **) malloc ((argc + 1) * sizeof (char *));
+
+		for (i = 0; i < term_argc; ++i)
+			argv[i] = term_argv[i];
+		if (term_argv != xterm_argv)
+			gnome_string_array_free (term_argv);
+
+		for (i = 0; i < item->exec_length; ++i)
+			argv[term_argc + i] = item->exec[i];
+
+		for (i = 0; i < the_argc; i++)
+			argv[term_argc + item->exec_length + i] = the_argv [i];
+		
+		argv[argc] = NULL;
+	} else {
+		if (the_argc != 0){
+			argc = the_argc + item->exec_length;
+			argv = (char **) malloc ((argc + 1) * sizeof (char *));
+
+			for (i = 0; i < item->exec_length; i++)
+				argv [i] = item->exec [i];
+			for (i = 0; i < the_argc; i++)
+				argv [item->exec_length + i] = the_argv [i];
+			argv [argc] = NULL;
+		} else {
+			argc = item->exec_length;
+			argv = item->exec;
+		}
+	}
+
+	uargv[0] = gnome_util_user_shell ();
+	uargv[1] = "-c";
+	uargv[2] = gnome_string_joinv (" ", argv);
+	uargv[3] = NULL;
+
+	/* FIXME: do something if there's an error.  */
+	gnome_execute_async (NULL, 4, uargv);
+
+	if (argv != item->exec)
+		g_free ((char *) argv);
+	g_free (uargv[0]);
+	g_free (uargv[2]);
 }
 
 void
 gnome_desktop_entry_launch (GnomeDesktopEntry *item)
 {
-	char *command;
+	gnome_desktop_entry_launch_with_args (item, 0, 0);
+}
 
-	g_assert (item != NULL);
+void
+gnome_desktop_entry_destroy (GnomeDesktopEntry *item)
+{
+      char *prefix;
 
-	if (item->terminal)
-		command = g_copy_strings ("(xterm -e \"", item->exec, "\") &", NULL);
-	else
-		command = g_copy_strings ("(true;", item->exec, ") &", NULL);
-
-	system (command);
-	g_free (command);
+      if (!item)
+	      return;
+      
+      prefix = g_copy_strings ("=", item->location, "=", NULL);
+      gnome_config_clean_file (prefix);
+      g_free (prefix);
+      gnome_desktop_entry_free (item);
 }
