@@ -6,7 +6,10 @@ BEGIN_GNOME_DECLS
 typedef struct {
 	char *name;		/* These contain the translated name/comment */
 	char *comment;
-	char *exec;		/* Program to execute */
+	int exec_length;	/* Length of `exec' vector.  Does not
+				   include the NULL terminator.  */
+	char **exec;		/* Command to execute.  Must be NULL
+				   terminated.  */
 	char *tryexec;		/* Test program to execute */
 	char *icon;		/* Icon name */
 	char *docpath;		/* Path to the documentation */
@@ -23,9 +26,11 @@ GnomeDesktopEntry *gnome_desktop_entry_load (char *file);
 GnomeDesktopEntry *gnome_desktop_entry_load_flags (char *file, int clean_from_memory_after_load);
 void gnome_desktop_entry_save (GnomeDesktopEntry *dentry);
 void gnome_desktop_entry_free (GnomeDesktopEntry *item);
+void gnome_desktop_entry_destroy (GnomeDesktopEntry *item);
 void gnome_desktop_entry_launch (GnomeDesktopEntry *item);
+void gnome_desktop_entry_launch_with_args (GnomeDesktopEntry *item, int the_argc, char *the_argv[]);
 
-int gnome_is_program_in_path (char *progname);
+char *gnome_is_program_in_path (char *progname);
 
 END_GNOME_DECLS
 
