@@ -5,6 +5,11 @@ typedef struct {
     gchar *path;
 } GnomeHelpMenuEntry;
 
+/* gnome_help_file_find_file() - Return a fully resolved file name for path. */
+/*                          User need to g_free path when done.         */
+/*                          Looks in LANG, then under C if not in LANG */
+gchar *gnome_help_file_find_file(gchar *app, gchar *path);
+
 /* gnome_help_file_path() - Return a fully resolved file name for path. */
 /*                          User need to g_free path when done.         */
 gchar *gnome_help_file_path(gchar *app, gchar *path);
