@@ -14,6 +14,7 @@
 #include <unistd.h>
 #include <glib.h>
 #include <sys/stat.h>
+#include <pwd.h>
 #include "gnome-defs.h"
 #include "gnome-util.h"
 
@@ -119,11 +120,11 @@ gnome_unconditional_datadir_file (char *filename)
 	return (gnome_dirrelative_file (GNOMEDATADIR, "share", filename, TRUE));
 }
 
-/* DOC: g_file_exists (char *filename)
+/* DOC: g_file_exists (const char *filename)
  * Returns true if filename exists
  */
 int
-g_file_exists (char *filename)
+g_file_exists (const char *filename)
 {
 	struct stat s;
 	
@@ -142,39 +143,41 @@ g_copy_strings (const char *first, ...)
 	char *data, *result;
 	
 	if (!first)
-		return NULL;
+		return 0;
 	
 	len = strlen (first);
 	va_start (ap, first);
 	
-	while ((data = va_arg (ap, char *)) != NULL)
+	while ((data = va_arg (ap, char *))!=0)
 		len += strlen (data);
 	
 	len++;
 	
-	result = (char *) g_malloc (len);
+	result = g_malloc (len);
 	va_end (ap);
 	va_start (ap, first);
 	strcpy (result, first);
-	while ((data = va_arg (ap, char *)) != NULL)
+	while ((data = va_arg (ap, char *)) != 0)
 		strcat (result, data);
 	va_end (ap);
 	
 	return result;
 }
 
-
 /* DOC: g_unix_error_string (int error_num)
  * Returns a pointer to a static location with a description of the errno
  */
-char *
+const gchar *
 g_unix_error_string (int error_num)
 {
-	static char buffer [256];
-	sprintf (buffer, "%s (%d)", g_strerror (error_num), error_num);
-	return buffer;
-}
+  static GString *buffer = NULL;
 
+  if(!buffer)
+    buffer = g_string_new(NULL);
+
+  g_string_sprintf (buffer, "%s (%d)", g_strerror (error_num), error_num);
+  return buffer->str;
+}
 
 /* DOC: g_concat_dir_and_file (const char *dir, const char *file)
  * returns a new allocated string that is the concatenation of dir and file,
@@ -183,74 +186,46 @@ g_unix_error_string (int error_num)
 char *
 g_concat_dir_and_file (const char *dir, const char *file)
 {
-	int l = strlen (dir);
-	
-	if (dir [l - 1] == PATH_SEP)
-		return g_copy_strings (dir, file, NULL);
-	else
+        /* If the directory name doesn't have a / on the end, we need
+	   to add one so we get a proper path to the file */
+	if (dir [strlen(dir) - 1] != PATH_SEP)
 		return g_copy_strings (dir, PATH_SEP_STR, file, NULL);
+	else
+		return g_copy_strings (dir, file, NULL);
 }
 
-
-/* returns the home directory of the user
- * This one is NOT to be free'd as it points into the 
- * env structure.
- * 
+/* DOC: gnome_util_user_shell ()
+ * Returns a newly allocated string that is the path to the user's
+ * preferred shell.
  */
-
 char *
-gnome_util_user_home(void)
+gnome_util_user_shell ()
 {
-	static char *home_dir;
-	static int init = 1;
-	
-	if (init) {
-		home_dir = getenv("HOME");
-		init = 0;
+	struct passwd *pw;
+	int i;
+	char *shell;
+	static char *shells [] = {
+		"/bin/bash", "/bin/zsh", "/bin/tcsh", "/bin/ksh",
+		"/bin/csh", "/bin/sh", 0
+	};
+
+	if ((shell = getenv ("SHELL"))){
+		return strdup (shell);
 	}
-	
-	return(home_dir);
-}
+	pw = getpwuid(getuid());
+	if (pw && pw->pw_shell) {
+		return strdup (pw->pw_shell);
+	} 
 
-char *gnome_util_prepend_user_config(char *file);
-char *gnome_util_user_config_file(char *file);
-
-char *
-gnome_util_user_config(void)
-{
-	static char *home_dir;
-	static int init = 1;
-	
-	if (init) {
-		home_dir = getenv("HOME");
-		init = 0;
+	for (i = 0; shells [i]; i++) {
+		if (g_file_exists (shells [i])){
+			return strdup (shells[i]);
+		}
 	}
-	
-	return(home_dir);
-}
 
-/* pass in a string, and it will add the users home dir ie,
- * pass in .gnome/bookmarks.html and it will return
- * /home/imain/.gnome/bookmarks.html
- * 
- * Remember to g_free() returned value! */
+	/* If /bin/sh doesn't exist, your system is truly broken.  */
+	abort ();
 
-char *
-gnome_util_prepend_user_home(char *file)
-{
-	return g_concat_dir_and_file(gnome_util_user_home(), file);
-}
-
-/* very similar to above, but adds $HOME/.gnome/ to beginning
- * This is meant to be the most useful version.
- */
-
-char *
-gnome_util_home_file(char *file)
-{
-	char *path = g_concat_dir_and_file(gnome_util_user_home(), ".gnome");
-	char *ret = g_copy_strings(path, "/", file, NULL);
-
-	g_free(path);
-	return ret;
+	/* Placate compiler.  */
+	return NULL;
 }
