@@ -5,6 +5,7 @@
 #include "libgnome/gnome-config.h"
 #include "libgnome/gnome-dentry.h"
 #include "libgnome/gnome-dl.h"
+#include "libgnome/gnome-exec.h"
 #include "libgnome/gnome-hook.h"
 #include "libgnome/gnome-i18n.h"
 #include "libgnome/gnome-parse.h"
@@ -15,6 +16,7 @@
 
 extern char *gnome_user_home_dir;
 extern char *gnome_user_dir;
+extern char *gnome_user_private_dir;
 extern char *gnome_app_id;
 
 void gnomelib_init (char *app_id);
