@@ -7,13 +7,14 @@
 #include <string.h>
 #include <sys/types.h>
 
-#include "gnome.h"
+#include "libgnome/libgnome.h"
 #include "gnome-client.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkprivate.h>
 #include <X11/Xatom.h>
 
 extern char *program_invocation_name;
+extern char *program_invocation_short_name;
 
 #ifndef HAVE_LIBSM
 typedef gpointer SmPointer;
@@ -94,9 +95,11 @@ static void   client_set_prop_from_array    (GnomeClient *client,
 static void   client_unset_prop             (GnomeClient *client,
 					     gchar       *prop_name);
 
+static void   client_unset_config_prefix    (GnomeClient *client);
+
 #endif /* HAVE_LIBSM */
 
-static void    array_free                    (gchar **array);
+#define array_free(array) gnome_string_array_free((array))
 static gchar** array_init_from_arg           (gint argc, 
 					      gchar *argv[]);
 static gchar** array_copy                    (gchar **source);
@@ -169,6 +172,8 @@ client_parse_func (int key, char *arg, struct argp_state *state)
       /* Found our argument.  Set the session id.  */
       g_assert (default_client != NULL);
       gnome_client_set_id (default_client, arg);
+      g_free (default_client->previous_id);
+      default_client->previous_id = g_strdup (arg);
     }
   else if (key == ARGP_KEY_SUCCESS)
     {
@@ -208,6 +213,11 @@ default_client_disconnect (GnomeClient *client,
 static void
 default_client_init (void)
 {
+#ifdef GTK_HAVE_SIGNAL_INIT
+  /* Make sure Gtk type system initialized.  */
+  gtk_type_init ();
+  gtk_signal_init ();
+#endif
   default_client= gnome_client_new_without_connection ();
 
   if (default_client)
@@ -232,10 +242,10 @@ gnome_client_new_default (void)
 /*****************************************************************************/
 /* GTK-class managing functions */
 
-guint
+GtkType
 gnome_client_get_type (void)
 {
-  static guint client_type = 0;
+  static GtkType client_type = 0;
   
   if (!client_type)
     {
@@ -266,7 +276,7 @@ gnome_client_class_init (GnomeClientClass *klass)
   
   client_signals[SAVE_YOURSELF] =
     gtk_signal_new ("save_yourself",
-		    GTK_RUN_FIRST,
+		    GTK_RUN_LAST,
 		    object_class->type,
 		    GTK_SIGNAL_OFFSET (GnomeClientClass, save_yourself),
 		    gnome_client_marshal_signal_1,
@@ -332,6 +342,8 @@ gnome_client_init (GnomeClient *client)
   client->client_id         = NULL;
   client->previous_id       = NULL;
   client->input_id          = 0;
+  client->config_prefix       = NULL;
+  client->global_config_prefix= NULL;
 
   /* Preset some default values.  */
   client->clone_command     = NULL;
@@ -389,8 +401,9 @@ gnome_real_client_destroy (GtkObject *object)
     gnome_client_disconnect (client);
 
   g_free (client->client_id);
-  if (client->previous_id)
-    g_free (client->previous_id);
+  g_free (client->previous_id);
+  g_free (client->config_prefix);
+  g_free (client->global_config_prefix);
 
   array_free (client->clone_command);
   g_free     (client->current_directory);
@@ -425,8 +438,7 @@ GnomeClient *
 gnome_client_new_without_connection (void)
 {
   GnomeClient *client;
-  gchar       *client_id;
-  
+
   client= gtk_type_new (gnome_client_get_type ());
 
   /* Preset the CloneCommand, RestartCommand and Program properties.
@@ -441,6 +453,22 @@ gnome_client_new_without_connection (void)
   return client;
 }
 
+void
+gnome_client_flush (GnomeClient *client)
+{
+#ifdef HAVE_LIBSM
+  IceConn conn;
+#endif
+
+  g_return_if_fail (client != NULL);
+  g_return_if_fail (GNOME_CLIENT_CONNECTED (client));
+
+#ifdef HAVE_LIBSM
+  conn = SmcGetIceConnection (client->smc_conn);
+  IceFlush (conn);
+#endif
+}
+
 /*****************************************************************************/
 
 #define ERROR_STRING_LENGTH 256
@@ -460,12 +488,12 @@ gnome_client_connect (GnomeClient *client)
 #ifdef HAVE_LIBSM
   if (GNOME_CLIENT_CONNECTED (client))
     return;
-  
+
   callbacks.save_yourself.callback      = client_save_yourself_callback;
   callbacks.die.callback                = client_die_callback;
   callbacks.save_complete.callback      = client_save_complete_callback;
   callbacks.shutdown_cancelled.callback = client_shutdown_cancelled_callback;
-  
+
   callbacks.save_yourself.client_data = 
     callbacks.die.client_data =
     callbacks.save_complete.client_data =
@@ -483,7 +511,7 @@ gnome_client_connect (GnomeClient *client)
 			   SmcShutdownCancelledProcMask,
 			   &callbacks,
 			   client->client_id, &client_id,
-			   ERROR_STRING_LENGTH, &error_string_ret);
+			   ERROR_STRING_LENGTH, error_string_ret);
       
       if (error_string_ret[0])
 	g_warning ("While connecting to session manager:\n%s.",
@@ -498,11 +526,15 @@ gnome_client_connect (GnomeClient *client)
       if (client->client_id == NULL)
 	{
 	  client->client_id = client_id;
+
+	  client_unset_config_prefix (client);
 	}
       else if (strcmp (client->client_id, client_id) != 0)
 	{
 	  g_free (client->client_id);
 	  client->client_id = client_id;
+
+	  client_unset_config_prefix (client);
 	}
       else
 	{
@@ -530,7 +562,9 @@ void
 gnome_client_disconnect (GnomeClient *client)
 {
   g_return_if_fail (client != NULL);
-  
+
+
+  gnome_client_flush (client);
   if (GNOME_CLIENT_CONNECTED (client))
     gtk_signal_emit (GTK_OBJECT (client), client_signals[DISCONNECT]);
 }
@@ -683,6 +717,8 @@ gnome_client_set_program (GnomeClient *client,
   g_free (client->program);
   
   client->program= g_strdup (program);
+
+  client_unset_config_prefix (client);  
 #ifdef HAVE_LIBSM
   client_set_prop_from_string (client, SmProgram, client->program);
 #endif /* HAVE_LIBSM */
@@ -737,7 +773,7 @@ gnome_client_set_restart_command (GnomeClient *client,
   /* Set the '--sm-client-id' option, if we are connected.  We set it
      in a copy, though, and not the original.  This lets us re-use the
      original for the clone command if we must.  */
-  if (GNOME_CLIENT_CONNECTED (client))
+  if (GNOME_CLIENT_CONNECTED (client) && client->restart_command)
     {
       gchar **dup = array_copy (client->restart_command);
       array_insert_sm_client_id_arg (&dup, client->client_id);
@@ -819,11 +855,13 @@ gnome_client_set_id (GnomeClient *client, const gchar *id)
   g_return_if_fail (client != NULL);
   g_return_if_fail (GNOME_IS_CLIENT (client));
   g_return_if_fail (!GNOME_CLIENT_CONNECTED (client));
-  
+
   g_return_if_fail (id != NULL);
 
   g_free (client->client_id);
   client->client_id= g_strdup (id);
+
+  client_unset_config_prefix (client);
 }
 
 
@@ -840,9 +878,82 @@ gchar *
 gnome_client_get_previous_id (GnomeClient *client)
 {
   g_return_val_if_fail (client != NULL, NULL);
+  g_return_val_if_fail (GNOME_IS_CLIENT (client), NULL);
+
   return client->previous_id;
 }
 
+
+static gchar *config_prefix= NULL;
+  
+gchar *
+gnome_client_get_config_prefix (GnomeClient *client)
+{
+  if (client == NULL)
+    {
+      if (!config_prefix)
+	config_prefix= g_copy_strings ("/", program_invocation_short_name, "/",
+				       NULL);
+
+      return config_prefix;
+    }
+
+  g_return_val_if_fail (GNOME_IS_CLIENT (client), NULL);
+  
+  if (!client->config_prefix)
+    {
+      char *name;
+      name= strrchr (client->program, '/');
+
+      name= name ? (name+1) : client->program;
+            
+      if (client->client_id)
+	client->config_prefix= g_copy_strings ("/", name, "-id#", 
+					       client->client_id, "/", NULL);
+      else
+	client->config_prefix= g_copy_strings ("/", name, "/", NULL);
+    }
+
+  return client->config_prefix;
+}
+
+gchar *
+gnome_client_get_global_config_prefix (GnomeClient *client)
+{
+  if (client == NULL)
+    {
+      if (!config_prefix)
+	config_prefix= g_copy_strings ("/", program_invocation_short_name, "/",
+				       NULL);
+
+      return config_prefix;
+    }
+
+  g_return_val_if_fail (GNOME_IS_CLIENT (client), NULL);
+  
+  if (!client->global_config_prefix)
+    {
+      char *name;
+      name= strrchr (client->program, '/');
+
+      name= name ? (name+1) : client->program;
+            
+      client->global_config_prefix= g_copy_strings ("/", name, "/", NULL);
+    }
+
+  return client->global_config_prefix;
+}
+
+static void
+client_unset_config_prefix (GnomeClient *client)
+{
+  g_free (client->config_prefix);
+  client->config_prefix= NULL;
+  g_free (client->global_config_prefix);
+  client->global_config_prefix= NULL;
+} 
+  
+
 /*****************************************************************************/
 
 static void
@@ -918,11 +1029,14 @@ gnome_real_client_connect (GnomeClient *client,
   client_set_prop_from_array (client, SmResignCommand,
 			      client->resign_command);
 
-  {
-    gchar **dup = array_copy (client->restart_command);
-    array_insert_sm_client_id_arg (&dup, client->client_id);
-    client_set_prop_from_array (client, SmRestartCommand, dup);
-  }
+  if (client->restart_command != NULL)
+    {
+      gchar **dup = array_copy (client->restart_command);
+      array_insert_sm_client_id_arg (&dup, client->client_id);
+      client_set_prop_from_array (client, SmRestartCommand, dup);
+    }
+  else
+    client_set_prop_from_array (client, SmRestartCommand, NULL);
 
   if (client->restart_style != -1)
     client_set_prop_from_gchar (client, SmRestartStyleHint,
@@ -1234,7 +1348,7 @@ client_set_prop_from_string (GnomeClient *client,
   prop.type     = SmARRAY8;
   prop.num_vals = 1;
   prop.vals     = &val;
-  val.length = strlen (value) + 1;
+  val.length = strlen (value);
   val.value  = value;
   proplist[0] = &prop;
   SmcSetProperties (client->smc_conn, 1, proplist);
@@ -1287,7 +1401,7 @@ client_set_prop_from_array (GnomeClient *client,
   vals = g_new (SmPropValue, argc);
   for (ptr = array, i = 0 ; i < argc ; ptr++, i++)
     {
-      vals[i].length = strlen (*ptr) + 1;
+      vals[i].length = strlen (*ptr);
       vals[i].value  = *ptr;
     }
 
@@ -1506,7 +1620,7 @@ array_insert_sm_client_id_arg (gchar ***array, gchar *client_id)
   gchar **ptr1;
   gchar **ptr2;
   gint    argc;
-      
+
   g_return_if_fail (client_id != NULL);
 
   for (ptr1 = *array, argc = 0; *ptr1 ; ptr1++ , argc++) /* LOOP */;
@@ -1530,22 +1644,6 @@ array_insert_sm_client_id_arg (gchar ***array, gchar *client_id)
 
 /*****************************************************************************/
 
-static void
-array_free (gchar **array)
-{
-  gchar **ptr;
-
-  if (array != NULL)
-    {
-      /* Free all entries in our array.  */
-      for (ptr = array ; *ptr ; ptr++)
-	g_free (*ptr);
-      
-      /* Free the array.  */
-      g_free (array);
-    }
-}
-
 static gchar **
 array_init_from_arg (gint argc, gchar *argv[])
 {
