@@ -8,11 +8,22 @@
 #include <gdk_imlib.h>
 #include "libgnome/libgnome.h"
 #include <argp.h>
+#include <signal.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include "gnome-preferences.h"
 
 extern char *program_invocation_name;
 extern char *program_invocation_short_name;
 
 static void gnome_rc_parse(gchar *command);
+static void gnome_segv_handle(int signum);
+static GdkPixmap *imlib_image_loader(GdkWindow   *window,
+				     GdkColormap *colormap,
+				     GdkBitmap  **mask,
+				     GdkColor    *transparent_color,
+				     const gchar *filename);
+
 
 
 
@@ -25,28 +36,17 @@ static void gnome_rc_parse(gchar *command);
    key number is used to look up the argument name we supply to Gtk.  */
 static struct argp_option our_gtk_options[] =
 {
-	{ "gdk-debug", -1, NULL, OPTION_HIDDEN,
-	  NULL, -2 },
-	{ "gdk-no-debug", -2, NULL, OPTION_HIDDEN,
-	  NULL, 0 },
-	{ "display", -3, N_("DISPLAY"), 0,
-	  N_("X display to use"), 0 },
-	{ "sync", -4, NULL, OPTION_HIDDEN,
-	  NULL, 0 },
-	{ "no-xshm", -5, NULL, 0,
-	  N_("Don't use X shared memory extension"), 0 },
-	{ "name", -6, N_("NAME"), 0,
-	  N_("FIXME"), 0 },
-	{ "class", -7, N_("CLASS"), 0,
-	  N_("FIXME"), 0 },
-	{ "gxid_host", -8, N_("HOST"), 0,
-	  N_("FIXME"), 0 },
-	{ "gxid_port", -9, N_("PORT"), 0,
-	  N_("FIXME"), 0 },
-	{ "xim-preedit", -10, N_("STYLE"), 0,
-	  N_("FIXME"), 0 },
-	{ "xim-status", -11, N_("STYLE"), 0,
-	  N_("FIXME"), 0 },
+	{ "gdk-debug",    -1, NULL,          OPTION_HIDDEN,    NULL, -2 },
+	{ "gdk-no-debug", -2, NULL,          OPTION_HIDDEN, NULL, 0 },
+	{ "display",      -3, N_("DISPLAY"), 0,	  N_("X display to use"), 0 },
+	{ "sync",         -4, NULL,          OPTION_HIDDEN, 	  NULL, 0 },
+	{ "no-xshm",      -5, NULL,          0,	  N_("Don't use X shared memory extension"), 0 },
+	{ "name",         -6, N_("NAME"),    0,	  N_("FIXME"), 0 },
+	{ "class",        -7, N_("CLASS"),   0,	  N_("FIXME"), 0 },
+	{ "gxid_host",    -8, N_("HOST"),    0,	  N_("FIXME"), 0 },
+	{ "gxid_port",    -9, N_("PORT"),    0,	  N_("FIXME"), 0 },
+	{ "xim-preedit", -10, N_("STYLE"),   0,	  N_("FIXME"), 0 },
+	{ "xim-status",  -11, N_("STYLE"),   0,	  N_("FIXME"), 0 },
 	{ NULL, 0, NULL, 0, NULL, 0 }
 };
 
@@ -70,30 +70,39 @@ our_gtk_parse_func (int key, char *arg, struct argp_state *state)
 						    our_gtk_options[- key - 1].name,
 						    NULL);
 		if (arg)
-			our_argv[our_argc++] = strdup (arg);
+			our_argv[our_argc++] = g_strdup (arg);
 	}
 	else if (key == ARGP_KEY_INIT)
 	{
-		our_argv = (char **) malloc ((state->argc + 1) * sizeof (char *));
+		/* We use twice the amount of space you'd think we need,
+		   because the user might write all options as
+		   `--foo=bar', but we always split into two arguments,
+		   like `-foo bar'.  */
+		our_argv = (char **) malloc (2 * (state->argc + 1) * sizeof (char *));
 		our_argc = 0;
-		our_argv[our_argc++] = strdup (state->argv[0]);
+		our_argv[our_argc++] = g_strdup (state->argv[0]);
 	}
 	else if (key == ARGP_KEY_SUCCESS)
 	{
 		int i, copy_ac = our_argc;
-		char **copy = (char **) malloc (our_argc * sizeof (char *));
+		char **copy = (char **) g_malloc (our_argc * sizeof (char *));
 
 		memcpy (copy, our_argv, our_argc * sizeof (char *));
 		our_argv[our_argc] = NULL;
 
 		gtk_init (&our_argc, &our_argv);
 		gdk_imlib_init ();
+		
+#ifdef GTK_HAVE_RC_SET_IMAGE_LOADER
+		/* New feature in 1.1 tree */
+		gtk_rc_set_image_loader(imlib_image_loader);
+#endif
 		gnome_rc_parse (program_invocation_name);
 
 		for (i = 0; i < copy_ac; ++i)
-			free (copy[i]);
-		free (copy);
-		free (our_argv);
+			g_free (copy[i]);
+		g_free (copy);
+		g_free (our_argv);
 	}
 	else
 		return ARGP_ERR_UNKNOWN;
@@ -136,6 +145,21 @@ gnome_init (char *app_id, struct argp *app_args,
 	    int argc, char **argv,
 	    unsigned int flags, int *arg_index)
 {
+	error_t retval;
+
+#if 0
+	struct sigaction sa;
+
+	/* Debugging segv when you have a pointer grab is less than
+	 * useful, Sop instead of removing the above #if 0, fix the DND
+	 */
+	/* Yes, we do this twice, so if an error occurs before init,
+	   it will be caught, and if it happens after init, we'll override
+	   gtk's handler */
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_handler = (gpointer)gnome_segv_handle;
+	sigaction(SIGSEGV, &sa, NULL);
+#endif
 	/* now we replace gtk_init() with gnome_init() in our apps */
 	gtk_set_locale();
 
@@ -154,11 +178,20 @@ gnome_init (char *app_id, struct argp *app_args,
 	
 	gnomelib_init (app_id);
 
+	/* Load UI preferences. Should this go here? */
+	gnome_preferences_load();
+
 	if (! argp_program_version_hook)
 		argp_program_version_hook = default_version_func;
 
 	/* Now parse command-line arguments.  */
-	return gnome_parse_arguments (app_args, argc, argv, flags, arg_index);
+	retval = gnome_parse_arguments (app_args, argc, argv, flags, arg_index);
+#if 0
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_handler = (gpointer)gnome_segv_handle;
+	sigaction(SIGSEGV, &sa, NULL);
+#endif
+	return retval;
 }
 
 /* perhaps this belongs in libgnome.. move it if you like. */
@@ -236,3 +269,69 @@ gnome_rc_parse (gchar *command)
 	
 	g_free (apprc);
 }
+
+static void gnome_segv_handle(int signum)
+{
+  static int in_segv = 0;
+  void *eip = NULL;
+  pid_t pid;
+
+#if 0
+  /* Is there any way to do this portably? */
+#ifdef linux
+#if defined(__i386__)
+  eip = sc->eip;
+#elif defined(__ppc__)
+  eip = sc->nc_ip;
+#elif defined(__sparc__)
+  eip = sc->sigc_pc;
+#endif
+#endif
+
+#endif /* #if 0 */
+
+  if(in_segv)
+    return;
+
+  in_segv++;
+
+  pid = fork();
+
+  if(pid)
+    {
+      int estatus;
+      pid_t eret;
+      eret = waitpid(pid, &estatus, 0);
+      /* We only do a zap-em if the user specifically requested it
+	 - if an error happens getting user decision, we carry on.
+         "Failsafe" is the idea */
+      if(WIFEXITED(estatus) && WEXITSTATUS(estatus) == 0)
+	exit(0);
+      else if(WIFEXITED(estatus) && WEXITSTATUS(estatus) == 99)
+	return; /* We don't decrement the in_segv lock
+		   - basically the user wants to ignore all future segv's */
+    }
+  else
+    {
+      char buf[32];
+      snprintf(buf, sizeof(buf), "%ld", eip);
+      /* Child process */
+      execl(GNOMEBINDIR "/gnome_segv", GNOMEBINDIR "/gnome_segv",
+	    program_invocation_name, buf, NULL);
+    }
+  in_segv--;
+}
+
+static GdkPixmap *
+imlib_image_loader(GdkWindow   *window,
+		   GdkColormap *colormap,
+		   GdkBitmap  **mask,
+		   GdkColor    *transparent_color,
+		   const gchar *filename)
+{
+  GdkPixmap *retval;
+  if(gdk_imlib_load_file_to_pixmap((char *)filename, &retval, mask))
+    return retval;
+  else
+    return NULL;
+}
