@@ -21,8 +21,11 @@ static void gnome_pixmap_size_request  (GtkWidget        *widget,
 					GtkRequisition   *requisition);
 static void gnome_pixmap_size_allocate (GtkWidget        *widget,
 					GtkAllocation    *allocation);
+static void gnome_pixmap_draw          (GtkWidget        *widget,
+					GdkRectangle     *area);
 static gint gnome_pixmap_expose        (GtkWidget        *widget,
 					GdkEventExpose   *event);
+static void setup_window_and_style     (GnomePixmap *gpixmap);
 
 
 static GtkWidgetClass *parent_class;
@@ -73,6 +76,7 @@ gnome_pixmap_class_init (GnomePixmapClass *class)
 	widget_class->realize = gnome_pixmap_realize;
 	widget_class->size_request = gnome_pixmap_size_request;
 	widget_class->size_allocate = gnome_pixmap_size_allocate;
+	widget_class->draw = gnome_pixmap_draw;
 	widget_class->expose_event = gnome_pixmap_expose;
 
 	object_class->destroy = gnome_pixmap_destroy;
@@ -113,6 +117,8 @@ gnome_pixmap_new_from_file (char *filename)
 {
 	GnomePixmap *gpixmap;
 
+	g_return_val_if_fail(filename != NULL, NULL);
+
 	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
 	gnome_pixmap_load_file (gpixmap, filename);
 
@@ -124,6 +130,8 @@ gnome_pixmap_new_from_file_at_size (char *filename, int width, int height)
 {
 	GnomePixmap *gpixmap;
 
+	g_return_val_if_fail(filename != NULL, NULL);
+
 	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
 	gnome_pixmap_load_file_at_size (gpixmap, filename, width, height);
 
@@ -135,6 +143,8 @@ gnome_pixmap_new_from_xpm_d (char **xpm_data)
 {
 	GnomePixmap *gpixmap;
 
+	g_return_val_if_fail(xpm_data != NULL, NULL);
+
 	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
 	gnome_pixmap_load_xpm_d (gpixmap, xpm_data);
 
@@ -146,6 +156,8 @@ gnome_pixmap_new_from_xpm_d_at_size (char **xpm_data, int width, int height)
 {
 	GnomePixmap *gpixmap;
 
+	g_return_val_if_fail(xpm_data != NULL, NULL);
+
 	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
 	gnome_pixmap_load_xpm_d_at_size (gpixmap, xpm_data, width, height);
 
@@ -158,6 +170,8 @@ gnome_pixmap_new_from_rgb_d (unsigned char *data, unsigned char *alpha,
 {
 	GnomePixmap *gpixmap;
 
+	g_return_val_if_fail(data != NULL, NULL);
+
 	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
 	gnome_pixmap_load_rgb_d (gpixmap, data, alpha,
 				 rgb_width, rgb_height);
@@ -166,16 +180,69 @@ gnome_pixmap_new_from_rgb_d (unsigned char *data, unsigned char *alpha,
 }
 
 GtkWidget *
-gnome_pixmap_new_from_rgb_d_at_size (char *data, unsigned char *alpha,
+gnome_pixmap_new_from_rgb_d_shaped (unsigned char *data, unsigned char *alpha,
+				    int rgb_width, int rgb_height,
+				    GdkImlibColor *shape_color)
+{
+	GnomePixmap *gpixmap;
+
+	g_return_val_if_fail(data != NULL, NULL);
+
+	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
+	gnome_pixmap_load_rgb_d_shaped (gpixmap, data, alpha,
+					rgb_width, rgb_height,
+					shape_color);
+
+	return GTK_WIDGET (gpixmap);
+}
+
+GtkWidget *
+gnome_pixmap_new_from_rgb_d_at_size (unsigned char *data, unsigned char *alpha,
 				     int rgb_width, int rgb_height,
 				     int width, int height)
 {
 	GnomePixmap *gpixmap;
 
+	g_return_val_if_fail(data != NULL, NULL);
+
 	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
 	gnome_pixmap_load_rgb_d_at_size (gpixmap, data, alpha,
-					 rgb_width, rgb_height,
-					 width, height);
+					rgb_width, rgb_height,
+					width, height);
+
+	return GTK_WIDGET (gpixmap);
+}
+
+GtkWidget *
+gnome_pixmap_new_from_gnome_pixmap (GnomePixmap *gpixmap_old)
+{
+	GnomePixmap *gpixmap;
+	GtkRequisition req;
+	GdkVisual *visual;
+	GdkGC *gc;
+
+	g_return_val_if_fail(gpixmap_old != NULL, NULL);
+	g_return_val_if_fail(GNOME_IS_PIXMAP(gpixmap_old), NULL);
+
+	gpixmap = gtk_type_new (gnome_pixmap_get_type ());
+	gtk_widget_size_request (GTK_WIDGET(gpixmap_old), &req);
+	if (GTK_WIDGET(gpixmap_old)->window)
+		visual = gdk_window_get_visual (GTK_WIDGET(gpixmap_old)->window);
+	else
+		visual = gdk_imlib_get_visual();
+	gpixmap->pixmap = gdk_pixmap_new (gpixmap_old->pixmap,
+					  req.width, req.height,
+					  visual->depth);
+	gc = gdk_gc_new (gpixmap->pixmap);
+	gdk_draw_pixmap (gpixmap->pixmap, gc, gpixmap_old->pixmap, 0, 0, 0, 0,
+			 req.width, req.height);
+	gdk_gc_destroy (gc);
+	gpixmap->mask = gdk_pixmap_new (gpixmap_old->mask,
+					req.width, req.height, 1);
+	gc = gdk_gc_new (gpixmap->mask);
+	gdk_draw_pixmap (gpixmap->mask, gc, gpixmap_old->mask, 0, 0, 0, 0,
+			 req.width, req.height);
+	gdk_gc_destroy (gc);
 
 	return GTK_WIDGET (gpixmap);
 }
@@ -283,30 +350,60 @@ gnome_pixmap_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 				 allocation->y + (allocation->height - widget->requisition.height) / 2);
 }
 
-static gint
-gnome_pixmap_expose (GtkWidget *widget, GdkEventExpose *event)
+static void
+paint (GnomePixmap *gpixmap, GdkRectangle *area)
+{
+	if (gpixmap->pixmap)
+		gdk_draw_pixmap (gpixmap->widget.window,
+				 gpixmap->widget.style->black_gc,
+				 gpixmap->pixmap,
+				 area->x, area->y,
+				 area->x, area->y,
+				 area->width, area->height);
+	else
+		gdk_window_clear_area (gpixmap->widget.window,
+				       area->x, area->y,
+				       area->width, area->height);
+}
+
+static void
+gnome_pixmap_draw (GtkWidget *widget, GdkRectangle *area)
 {
 	GnomePixmap *gpixmap;
+	GdkRectangle w_area;
+	GdkRectangle p_area;
 
-	g_return_val_if_fail (widget != NULL, FALSE);
-	g_return_val_if_fail (GNOME_IS_PIXMAP (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (GNOME_IS_PIXMAP (widget));
+	g_return_if_fail (area != NULL);
 
 	if (GTK_WIDGET_DRAWABLE (widget)) {
 		gpixmap = GNOME_PIXMAP (widget);
 
-		if (gpixmap->pixmap)
-			gdk_draw_pixmap (widget->window,
-					 widget->style->black_gc,
-					 gpixmap->pixmap,
-					 event->area.x, event->area.y,
-					 event->area.x, event->area.y,
-					 event->area.width, event->area.height);
-		else
-			gdk_window_clear_area (widget->window,
-					       event->area.x, event->area.y,
-					       event->area.width, event->area.height);
+		/* Offset the area because the window does not fill the allocation */
+
+		area->x -= (widget->allocation.width - widget->requisition.width) / 2;
+		area->y -= (widget->allocation.height - widget->requisition.height) / 2;
+
+		w_area.x = 0;
+		w_area.y = 0;
+		w_area.width = widget->requisition.width;
+		w_area.height = widget->requisition.height;
+
+		if (gdk_rectangle_intersect (area, &w_area, &p_area))
+			paint (gpixmap, &p_area);
 	}
+}
+
+static gint
+gnome_pixmap_expose (GtkWidget *widget, GdkEventExpose *event)
+{
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (GNOME_IS_PIXMAP (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	if (GTK_WIDGET_DRAWABLE (widget))
+		paint (GNOME_PIXMAP (widget), &event->area);
 
 	return FALSE;
 }
@@ -327,8 +424,15 @@ finish_load (GnomePixmap *gpixmap, GdkImlibImage *im, int scaled, int width, int
 
 	gdk_imlib_destroy_image (im);
 
-	if (GTK_WIDGET_REALIZED (gpixmap))
+	if (GTK_WIDGET_REALIZED (gpixmap)) {
+		if (GTK_WIDGET_MAPPED (gpixmap))
+			gdk_window_hide (GTK_WIDGET (gpixmap)->window);
+
 		setup_window_and_style (gpixmap);
+	}
+
+	if (GTK_WIDGET_MAPPED (gpixmap))
+		gdk_window_show (GTK_WIDGET (gpixmap)->window);
 
 	if (GTK_WIDGET_VISIBLE (gpixmap))
 		gtk_widget_queue_resize (GTK_WIDGET (gpixmap));
@@ -369,6 +473,22 @@ load_rgb_d (GnomePixmap *gpixmap, unsigned char *data, unsigned char *alpha,
 	finish_load (gpixmap, im, scaled, width, height);
 }
 
+static void
+load_rgb_d_shaped (GnomePixmap *gpixmap, unsigned char *data,
+		   unsigned char *alpha,
+		   int rgb_width, int rgb_height,
+		   int scaled, int width, int height,
+		   GdkImlibColor *shape_color)
+{
+	GdkImlibImage *im;
+
+	free_pixmap_and_mask (gpixmap);
+
+	im = gdk_imlib_create_image_from_data (data, alpha, rgb_width, rgb_height);
+	gdk_imlib_set_image_shape (im, shape_color);
+	finish_load (gpixmap, im, scaled, width, height);
+}
+
 void
 gnome_pixmap_load_file (GnomePixmap *gpixmap, char *filename)
 {
@@ -426,6 +546,22 @@ gnome_pixmap_load_rgb_d (GnomePixmap *gpixmap, unsigned char *data, unsigned cha
 	load_rgb_d (gpixmap, data, alpha, rgb_width, rgb_height, FALSE, 0, 0);
 }
 
+void
+gnome_pixmap_load_rgb_d_shaped (GnomePixmap *gpixmap, unsigned char *data,
+				unsigned char *alpha,
+				int rgb_width, int rgb_height,
+				GdkImlibColor *shape_color)
+{
+	g_return_if_fail (gpixmap != NULL);
+	g_return_if_fail (GNOME_IS_PIXMAP (gpixmap));
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (rgb_width > 0);
+	g_return_if_fail (rgb_height > 0);
+
+	load_rgb_d_shaped (gpixmap, data, alpha, rgb_width, rgb_height, FALSE,
+			   0, 0, shape_color);
+}
+
 void
 gnome_pixmap_load_rgb_d_at_size (GnomePixmap *gpixmap, unsigned char *data, unsigned char *alpha,
 				 int rgb_width, int rgb_height,
@@ -507,6 +643,8 @@ load_pixmap(GdkWindow *window, GdkPixmap **pixmap, GdkBitmap **mask, GdkColor *t
 void
 gnome_create_pixmap_gdk (GdkWindow *window, GdkPixmap **pixmap, GdkBitmap **mask, GdkColor *transparent, char *file)
 {
+	printf ("gnome_create_pixmap_gdk: this function is obsolete and evil; please use gdk_imlib instead\n");
+
 	g_assert(window != NULL);
 	g_assert(pixmap != NULL);
 	g_assert(mask != NULL);
@@ -526,6 +664,8 @@ gnome_create_pixmap_gtk (GtkWidget *window, GdkPixmap **pixmap, GdkBitmap **mask
 {
 	GtkStyle *style;
 	
+	printf ("gnome_create_pixmap_gtk: this function is obsolete and evil; please use gdk_imlib instead\n");
+
 	g_assert(window != NULL);
 	g_assert(pixmap != NULL);
 	g_assert(mask != NULL);
@@ -553,6 +693,8 @@ gnome_create_pixmap_gtk_d (GtkWidget *window, GdkPixmap **pixmap, GdkBitmap **ma
 {
 	GtkStyle *style;
 	
+	printf ("gnome_create_pixmap_gtk_d: this function is obsolete and evil; please use gdk_imlib instead\n");
+
 	g_assert(window != NULL);
 	g_assert(pixmap != NULL);
 	g_assert(mask != NULL);
@@ -572,6 +714,8 @@ gnome_create_pixmap_widget (GtkWidget *window, GtkWidget *holder, char *file)
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 
+	printf ("gnome_create_pixmap_widget: this function is obsolete and evil; please use GnomePixmap instead\n");
+
 	if (!file || !g_file_exists(file))
 		return NULL;
 
@@ -586,6 +730,8 @@ gnome_create_pixmap_widget_d (GtkWidget *window, GtkWidget *holder, char **data)
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 
+	printf ("gnome_create_pixmap_widget_d: this function is obsolete and evil; please use GnomePixmap instead\n");
+
 	gnome_create_pixmap_gtk_d(window, &pixmap, &mask, holder, data);
 
 	return gtk_pixmap_new(pixmap, mask);
@@ -596,6 +742,8 @@ gnome_set_pixmap_widget (GtkPixmap *pixmap, GtkWidget *window, GtkWidget *holder
 {
 	GdkPixmap *gpixmap;
 	GdkBitmap *mask;
+
+	printf ("gnome_set_pixmap_widget: this function is obsolete and evil; please use gdk_imlib instead\n");
 	
 	g_assert (pixmap != NULL);
 
@@ -613,6 +761,8 @@ gnome_set_pixmap_widget_d (GtkPixmap *pixmap, GtkWidget *window, GtkWidget *hold
 	GdkPixmap *gpixmap;
 	GdkBitmap *mask;
 	
+	printf ("gnome_set_pixmap_widget_d: this function is obsolete and evil; please use gdk_imlib instead\n");
+	
 	g_assert (pixmap != NULL);
 
 	gnome_create_pixmap_gtk_d(window, &gpixmap, &mask, holder, data);
