@@ -6,7 +6,8 @@
  * Licensed under the terms of the GNU LGPL
  */
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+#include "libgnome/libgnome.h"
 #include <string.h>
 #include "gtk-ted.h"
 
@@ -44,8 +45,6 @@ typedef struct {
 
 static char *app_name;
 
-static GtkTableClass *parent_class = NULL;
-
 static void
 gtk_ted_destroy (GtkObject *object)
 {
@@ -173,10 +172,6 @@ gtk_ted_attach (GtkTed *ted, GtkWidget *widget, struct ted_widget_info *wi)
 			  1+ wi->start_col * 2, 1+ (wi->start_col * 2)+(wi->col_span * 2)-1,
 			  1+ wi->start_row * 2, 1+ (wi->start_row * 2)+(wi->row_span * 2)-1,
 			  GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
-/* 
-			  wi->flags_x,
-			  wi->flags_y,
-*/
 			  0, 0);
 
 	gtk_ted_align_new (align, wi);
@@ -314,6 +309,7 @@ lift_me (GtkWidget *w, GdkEventButton *event, GtkWidget *target)
 	gtk_widget_show (top);
 	gdk_pointer_grab (w->window, FALSE, GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
 			  NULL, NULL, GDK_CURRENT_TIME);
+	gtk_signal_emit_stop_by_name (GTK_OBJECT (w), "button_press_event");
 }
 
 /*
@@ -790,15 +786,17 @@ gtk_ted_control_bar_new (GtkTed *ted)
 	return hbox;
 }
 
-void
+static void
+gtk_ted_init_divisions (GtkTed *ted)
+{
+	gtk_ted_add_controls (ted, ted->top_col+1, ted->top_row+1);
+	gtk_ted_add_divisions (ted, ted->top_col+1, ted->top_row+1);
+}
+
+static void
 gtk_ted_init_editor (GtkTed *ted)
 {
 	GtkWidget *control_window, *control_bar;
-	int rows = 6;
-	int cols = 6;
-	
-	gtk_ted_add_controls (ted, cols, rows);
-	gtk_ted_add_divisions (ted, cols, rows);	
 
 	control_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	ted->widget_box = gtk_table_new (TED_TOP_ROW+2, TED_TOP_COL, 0);
@@ -1298,7 +1296,7 @@ gtk_ted_widget_control_new (GtkTed *ted, GtkWidget *widget, char *name)
 void
 gtk_ted_add (GtkTed *ted, GtkWidget *widget, char *original_name)
 {
-	GtkWidget *align, *event, *l;
+	GtkWidget *align;
 	struct ted_widget_info *wi;
 	char *name = g_strdup (original_name), *p;
 
@@ -1333,7 +1331,8 @@ gtk_ted_prepare_widgets_edit (gpointer key, gpointer value, gpointer user_data)
 {
 	GtkTed *ted = user_data;
 	struct ted_widget_info *wi = value;
-
+	int top_col, top_row;
+	
 	/* This is a widget that had layout information, but does not exist in this
 	 * instance of the execution
 	 */
@@ -1342,6 +1341,10 @@ gtk_ted_prepare_widgets_edit (gpointer key, gpointer value, gpointer user_data)
 	
 	gtk_ted_prepare_editable_widget (wi, GTK_WIDGET (ted), key);
 	gtk_ted_widget_control_new (ted, wi->widget, key);
+	top_col = wi->start_col + wi->col_span;
+	top_row = wi->start_row + wi->row_span;
+	ted->top_col = MAX (ted->top_col, wi->start_col + wi->col_span);
+	ted->top_row = MAX (ted->top_row, wi->start_row + wi->row_span);
 }
 
 static void
@@ -1349,17 +1352,60 @@ gtk_ted_setup_layout (gpointer key, gpointer value, gpointer user_data)
 {
 	GtkTed *ted = user_data;
 	struct ted_widget_info *wi = value;
+	int top_col, top_row;
 
+	/* If the widget does no longer exist, but it is present on the layout
+	 * file, return
+	 */
+	if (!wi->widget)
+		return;
+	
+	top_col = wi->start_col + wi->col_span;
+	top_row = wi->start_row + wi->row_span;
+	
 	gtk_table_attach (GTK_TABLE (ted), wi->widget,
-			  wi->start_col, wi->start_col + wi->col_span,
-			  wi->start_row, wi->start_row + wi->row_span,
+			  wi->start_col, top_col,
+			  wi->start_row, top_row, 
 			  GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
-/*			  wi->flags_x, wi->flags_y, 0, 0); */
+	
+	if (top_col > ted->top_col)
+		ted->top_col = top_col;
+	if (top_row > ted->top_row)
+		ted->top_row = top_row;
+	
 	gtk_ted_align_new (wi->widget, wi);
 	ted->top_col = MAX (ted->top_col, wi->start_col + wi->col_span);
 	ted->top_row = MAX (ted->top_row, wi->start_row + wi->row_span);
 }
 
+void
+gtk_ted_set_spacings (GtkTed *ted)
+{
+	int i;
+	
+	for (i = 0; i < ted->top_col; i++){
+		GtkWidget *spacer;
+
+		spacer = gtk_hbox_new (0, 0);
+		gtk_widget_set_usize (spacer, 5, 5);
+		gtk_table_attach (GTK_TABLE (ted), spacer,
+				  i, i+1, 0, 1, 0, 0, 0, 0);
+		gtk_widget_show (spacer);
+	}
+	
+	for (i = 0; i < ted->top_row; i++){
+		GtkWidget *spacer;
+
+		spacer = gtk_hbox_new (0, 0);
+		gtk_widget_set_usize (spacer, 5, 5);
+		gtk_table_attach (GTK_TABLE (ted), spacer,
+				  0, 1, i, i+1, 0, 0, 0, 0);
+		gtk_widget_show (spacer);
+	}
+	gtk_table_set_row_spacings (GTK_TABLE (ted), 5);
+	gtk_table_set_col_spacings (GTK_TABLE (ted), 5);
+}
+
 void
 gtk_ted_prepare (GtkTed *ted)
 {
@@ -1373,10 +1419,11 @@ gtk_ted_prepare (GtkTed *ted)
 		gtk_ted_init_editor (ted);
 		ted->in_gui = 1;
 		g_hash_table_foreach (ted->widgets, gtk_ted_prepare_widgets_edit, ted);
+		gtk_ted_init_divisions (ted);
 	} else {
-		gtk_table_set_row_spacings (GTK_TABLE (ted), 5);
-		gtk_table_set_col_spacings (GTK_TABLE (ted), 5);
+		ted->top_col = ted->top_row = 0;
 		g_hash_table_foreach (ted->widgets, gtk_ted_setup_layout, ted);
+		gtk_ted_set_spacings (ted);
 	}
 }
 
