@@ -26,7 +26,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 
-
 /* --- limits & defaults --- */
 #define	BLINK_TIMEOUT			(333)
 #define	REFRESH_TIMEOUT			(25 * 0 /* FIXME ;) */)
@@ -125,7 +124,7 @@ static	void	gtk_entry_selection_handler	(GtkWidget	*widget,
 /* --- static variables --- */
 
 static GtkWidgetClass	*parent_class = NULL;
-static gint		 term_signals[LAST_SIGNAL] = { 0 };
+static guint		 term_signals[LAST_SIGNAL] = { 0 };
 static gchar		 gtk_term_blank_char = ' ';
 static GtkTermAttrib	 gtk_term_blank_attrib = { FLAG_NONE, GTK_TERM_MAX_COLORS - 1, 0 };
 
@@ -137,10 +136,10 @@ static GtkTermAttrib	 gtk_term_blank_attrib = { FLAG_NONE, GTK_TERM_MAX_COLORS -
 
 /* --- functions --- */
 
-guint
+GtkType
 gtk_term_get_type ()
 {
-  static guint term_type = 0;
+  static GtkType term_type = 0;
   
   if (!term_type)
   {
@@ -891,7 +890,9 @@ gtk_term_button_press (GtkWidget      *widget,
   term = GTK_TERM (widget);
   x = event->x / term->char_width;
   y = event->y / term->char_height;
-  
+
+  if (x >= term->term_width) x = term->term_width - 1;
+  if (y >= term->term_height + term->first_line) y = term->term_height + term->first_line - 1;
   
   gtk_term_selection_clear (widget, NULL);
   
@@ -911,7 +912,7 @@ gtk_term_button_press (GtkWidget      *widget,
       break;
       
     case GDK_2BUTTON_PRESS:
-      printf ("FIXME: need a word selection\n");
+      g_print ("FIXME: need a word selection\n");
       break;
       
     case GDK_3BUTTON_PRESS:
@@ -1067,7 +1068,7 @@ gtk_term_motion_notify (GtkWidget      *widget,
   term->sel_e_y = y;
   
   /*
-    printf ("(%u, %u, %u, %u)\n", term->sel_b_x, term->sel_b_y, term->sel_e_x, term->sel_e_y);
+    g_print ("(%u, %u, %u, %u)\n", term->sel_b_x, term->sel_b_y, term->sel_e_x, term->sel_e_y);
     */
   
   return FALSE;
@@ -1239,7 +1240,7 @@ gtk_entry_selection_handler (GtkWidget		*widget,
 			    sizeof (gchar) * 8,
 			    term->sel_buffer,
 			    term->sel_len);
-    printf ("selction: \"%s\"\n", term->sel_buffer);
+    g_print ("selction: \"%s\"\n", term->sel_buffer);
   }
 }
 
